#ifndef RECORD_H
#define RECORD_H

#include <map>
#include <deque>
#include <string>
#include <iostream>
#include <fstream>
#include <sys/resource.h>
#include <mpi.h>
#include "cca.h"
#include "ports/Measurement_CCA.h"
#include "ports/MPIBorrow.h"

#define MPI_ID 1

using namespace std;

namespace performance{
  class Record{
  private:

    int rank;	  
    string name, filename;
    map<string,Timer*> timers;
    map<string, double> parameters;
    map<string, double> last_data_exclusive;
    map<string, double> last_data_inclusive;
    map<string, double> mpi_last_data;
    map<string, double>mpi_stop_data;
    deque<map<string,double> > t_data;
    bool timer_created;
    bool pnames_set;
    bool filename_exists;
    performance::ccaports::Measurement * measurement_m;
    classic::gov::cca::Services * svc;

    //routines
    int createTimer(string tname);
    int startTimer(string tname);
    int stopTimer(string tname);

  public:
    //constructor, initialize variables to zero
    Record(string n,classic::gov::cca::Services * fs, performance::ccaports::Measurement * m);
    ~Record();
    
    /****************************************************************
     *  Routines
     ***************************************************************/
    int setRank(int r);
    int getRank();
    //start and stop timing will create a timer with the same name as the
    //record object and start and stop it.  The bool parameter specifies
    //whether we should start and stop the MPI timers with the user
    //timer.  The coinciding start and stop timing calls must match (e.g.
    //if you call startTiming(true), you must stop with stopTiming(true))
    //The second input in the stop call is a reference to a map where the 
    //counters can be stored
    int startTiming(bool mpitimers);
    int stopTiming(bool mpitimers, map<string,double> &pc);
    int startMPITimers();
    int stopMPITimers();
    //Set the component's parameters.  
    int setParameters(map<string,double> &m);
    //Set the filename to dump out data.  Can only set once.  If attempted 
    //to be set more than once, the new name is ignored and a -1 is returned.
    //1 is returned on success.
    int setFileName(string s);
    string getFileName();
    //Dump the measurements stored in the performance_data map and timers 
    //map to file.  Default name is dump.out, if not set using 
    //setFileName();
    int dumpToFile(string rname);
  };
}
#endif
