#include <iostream>
#include <mpi.h>

#include "decaf_Framework.hh"
#include "cca_ComponentID.hh"
#include "cca_ports_GoPort.hh"

int main(int argc, char *argv[]) 
{

  MPI_Init(&argc, &argv);

  // Create the decaf framework
  decaf::Framework fwk = decaf::Framework::_create();

  // Create instances of the components used
  cca::ComponentID integrator = 
    fwk.createInstance( "ParallelIntegratorInstance", 
			"integrators.ParallelIntegrator",
			0);
  
  cca::ComponentID function = 
    fwk.createInstance( "PiFunctionInstance",
			"functions.PiFunction",
			0);

  cca::ComponentID driver = 
    fwk.createInstance( "DriverInstance",
			"tutorial.Driver",
			0);
  
  // Connect the components

  fwk.connect (driver, "IntegratorPort",		// user
	       integrator, "IntegratorPort");		// provider

  fwk.connect (integrator, "FunctionPort",		// user
	       function, "FunctionPort");		// provider


  // Execute the drivercomponent's go method
  cca::ports::GoPort go = fwk.lookupPort(driver,"Go");
  go.go();

  MPI_Finalize();
  return 0;
}
