// 
// File:          sidl_BaseClass.cc
// Symbol:        sidl.BaseClass-v0.9.0
// Symbol Type:   class
// Babel Version: 0.9.3
// Release:       $Name:  $
// Revision:      @(#) $Id: $
// Description:   Client-side glue code for sidl.BaseClass
// 
// Copyright (c) 2000-2002, The Regents of the University of California.
// Produced at the Lawrence Livermore National Laboratory.
// Written by the Components Team <components@llnl.gov>
// All rights reserved.
// 
// This file is part of Babel. For more information, see
// http://www.llnl.gov/CASC/components/. Please read the COPYRIGHT file
// for Our Notice and the LICENSE file for the GNU Lesser General Public
// License.
// 
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License (as published by
// the Free Software Foundation) version 2.1 dated February 1999.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms and
// conditions of the GNU Lesser General Public License for more details.
// 
// You should have recieved a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software Foundation,
// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
// 
// WARNING: Automatically generated; changes will be lost
// 
// babel-version = 0.9.3
// 

#ifndef included_sidl_BaseClass_hh
#include "sidl_BaseClass.hh"
#endif
#ifndef included_sidl_BaseInterface_hh
#include "sidl_BaseInterface.hh"
#endif
#ifndef included_sidl_BaseClass_hh
#include "sidl_BaseClass.hh"
#endif
#include "sidl_String.h"


//////////////////////////////////////////////////
// 
// User Defined Methods
// 


/**
 * <p>
 * Add one to the intrinsic reference count in the underlying object.
 * Object in <code>sidl</code> have an intrinsic reference count.
 * Objects continue to exist as long as the reference count is
 * positive. Clients should call this method whenever they
 * create another ongoing reference to an object or interface.
 * </p>
 * <p>
 * This does not have a return value because there is no language
 * independent type that can refer to an interface or a
 * class.
 * </p>
 */
void
sidl::BaseClass::addRef(  )
throw ( ::sidl::NullIORException ) 

{
  if ( d_self == 0 ) {
    throw ::sidl::NullIORException( ::std::string (
      "Null IOR Pointer in \"sidl::BaseClass::addRef()\""
    ));
  }

  if ( !d_weak_reference ) {
    // pack args to dispatch to ior

    // dispatch to ior
    (*(d_self->d_epv->f_addRef))(d_self );
    // unpack results and cleanup

  }
}



/**
 * Decrease by one the intrinsic reference count in the underlying
 * object, and delete the object if the reference is non-positive.
 * Objects in <code>sidl</code> have an intrinsic reference count.
 * Clients should call this method whenever they remove a
 * reference to an object or interface.
 */
void
sidl::BaseClass::deleteRef(  )
throw ( ::sidl::NullIORException ) 

{
  if ( d_self == 0 ) {
    throw ::sidl::NullIORException( ::std::string (
      "Null IOR Pointer in \"sidl::BaseClass::deleteRef()\""
    ));
  }

  if ( !d_weak_reference ) {
    // pack args to dispatch to ior

    // dispatch to ior
    (*(d_self->d_epv->f_deleteRef))(d_self );
    // unpack results and cleanup

    d_self = 0;
  }
}



/**
 * Return true if and only if <code>obj</code> refers to the same
 * object as this object.
 */
bool
sidl::BaseClass::isSame( /*in*/ ::sidl::BaseInterface iobj )
throw ( ::sidl::NullIORException ) 

{
  if ( d_self == 0 ) {
    throw ::sidl::NullIORException( ::std::string (
      "Null IOR Pointer in \"sidl::BaseClass::isSame()\""
    ));
  }
  bool _result;
  // pack args to dispatch to ior
  sidl_bool _local_result;
  // dispatch to ior
  _local_result = (*(d_self->d_epv->f_isSame))(d_self,
    /* in */ iobj._get_ior() );
  // unpack results and cleanup
  _result = _local_result;
  return _result;
}



/**
 * Check whether the object can support the specified interface or
 * class.  If the <code>sidl</code> type name in <code>name</code>
 * is supported, then a reference to that object is returned with the
 * reference count incremented.  The callee will be responsible for
 * calling <code>deleteRef</code> on the returned object.  If
 * the specified type is not supported, then a null reference is
 * returned.
 */
::sidl::BaseInterface
sidl::BaseClass::queryInt( /*in*/ const ::std::string& name )
throw ( ::sidl::NullIORException ) 

{
  if ( d_self == 0 ) {
    throw ::sidl::NullIORException( ::std::string (
      "Null IOR Pointer in \"sidl::BaseClass::queryInt()\""
    ));
  }
  ::sidl::BaseInterface _result;
  // pack args to dispatch to ior

  // dispatch to ior
  _result = ::sidl::BaseInterface( (*(d_self->d_epv->f_queryInt))(d_self,
    /* in */ name.c_str() ), false);
  // unpack results and cleanup

  return _result;
}



/**
 * Return whether this object is an instance of the specified type.
 * The string name must be the <code>sidl</code> type name.  This
 * routine will return <code>true</code> if and only if a cast to
 * the string type name would succeed.
 */
bool
sidl::BaseClass::isType( /*in*/ const ::std::string& name )
throw ( ::sidl::NullIORException ) 

{
  if ( d_self == 0 ) {
    throw ::sidl::NullIORException( ::std::string (
      "Null IOR Pointer in \"sidl::BaseClass::isType()\""
    ));
  }
  bool _result;
  // pack args to dispatch to ior
  sidl_bool _local_result;
  // dispatch to ior
  _local_result = (*(d_self->d_epv->f_isType))(d_self, /* in */ name.c_str() );
  // unpack results and cleanup
  _result = _local_result;
  return _result;
}



/**
 * Return the meta-data about the class implementing this interface.
 */
::sidl::ClassInfo
sidl::BaseClass::getClassInfo(  )
throw ( ::sidl::NullIORException ) 

{
  if ( d_self == 0 ) {
    throw ::sidl::NullIORException( ::std::string (
      "Null IOR Pointer in \"sidl::BaseClass::getClassInfo()\""
    ));
  }
  ::sidl::ClassInfo _result;
  // pack args to dispatch to ior

  // dispatch to ior
  _result = ::sidl::ClassInfo( (*(d_self->d_epv->f_getClassInfo))(d_self ),
    false);
  // unpack results and cleanup

  return _result;
}



//////////////////////////////////////////////////
// 
// End User Defined Methods
// (everything else in this file is specific to
//  Babel's C++ bindings)
// 

// static constructor
::sidl::BaseClass
sidl::BaseClass::_create() {
  ::sidl::BaseClass self( (*_get_ext()->createObject)(), false );
  return self;
}

// default destructor
sidl::BaseClass::~BaseClass () {
  if ( d_self != 0 ) {
    deleteRef();
  }
}

// copy constructor
sidl::BaseClass::BaseClass ( const ::sidl::BaseClass& original ) {
  d_self = const_cast< ior_t*>(original.d_self);
  d_weak_reference = original.d_weak_reference;
  if (d_self != 0 ) {
    addRef();
  }
}

// assignment operator
::sidl::BaseClass&
sidl::BaseClass::operator=( const ::sidl::BaseClass& rhs ) {
  if ( d_self != rhs.d_self ) {
    if ( d_self != 0 ) {
      deleteRef();
    }
    d_self = const_cast< ior_t*>(rhs.d_self);
    d_weak_reference = rhs.d_weak_reference;
    if ( d_self != 0 ) {
      addRef();
    }
  }
  return *this;
}

// conversion from ior to C++ class
sidl::BaseClass::BaseClass ( ::sidl::BaseClass::ior_t* ior ) 
    : d_self( ior ), d_weak_reference(false) {
  if ( d_self != 0 ) {
    addRef();
  }
}

// Alternate constructor: does not call addRef()
// (sets d_weak_reference=isWeak)
// For internal use by Impls (fixes bug#275)
sidl::BaseClass::BaseClass ( ::sidl::BaseClass::ior_t* ior, bool isWeak ) 
    : d_self( ior ), d_weak_reference(isWeak) { 
}

// conversion from a StubBase
sidl::BaseClass::BaseClass ( const ::sidl::StubBase& base )
{
  d_self = reinterpret_cast< ior_t*>(base._cast("sidl.BaseClass"));
  d_weak_reference = false;
  if (d_self != 0) {
    addRef();
  }
}

// protected method that implements casting
void* sidl::BaseClass::_cast(const char* type) const
{
  void* ptr = 0;
  if ( d_self != 0 ) {
    ptr = reinterpret_cast< void*>((*d_self->d_epv->f__cast)(d_self, type));
  }
  return ptr;
}

// Static data type
const ::sidl::BaseClass::ext_t * sidl::BaseClass::s_ext = 0;

// private static method to get static data type
const ::sidl::BaseClass::ext_t *
sidl::BaseClass::_get_ext()
  throw (::sidl::NullIORException)
{
  if (! s_ext ) {
    s_ext = sidl_BaseClass__externals();
  }
  return s_ext;
}

