// 
// File:          SIDL_Loader.hh
// Symbol:        SIDL.Loader-v0.8.2
// Symbol Type:   class
// Babel Version: 0.8.4
// Release:       $Name:  $
// Revision:      @(#) $Id: $
// Description:   Client-side glue code for SIDL.Loader
// 
// Copyright (c) 2000-2002, The Regents of the University of California.
// Produced at the Lawrence Livermore National Laboratory.
// Written by the Components Team <components@llnl.gov>
// All rights reserved.
// 
// This file is part of Babel. For more information, see
// http://www.llnl.gov/CASC/components/. Please read the COPYRIGHT file
// for Our Notice and the LICENSE file for the GNU Lesser General Public
// License.
// 
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License (as published by
// the Free Software Foundation) version 2.1 dated February 1999.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms and
// conditions of the GNU Lesser General Public License for more details.
// 
// You should have recieved a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software Foundation,
// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
// 
// WARNING: Automatically generated; changes will be lost
// 
// babel-version = 0.8.4
// 

#ifndef included_SIDL_Loader_hh
#define included_SIDL_Loader_hh

// declare class before #includes
// (this alleviates circular #include guard problems)[BUG#393]
namespace SIDL { 

  class Loader;
} // end namespace SIDL

// Some compilers need to define array template before the specializations
#ifndef included_SIDL_cxx_hh
#include "SIDL_cxx.hh"
#endif
namespace SIDL {
  template<>
  class array< ::SIDL::Loader>;
}

#ifndef included_SIDL_cxx_hh
#include "SIDL_cxx.hh"
#endif
#ifndef included_SIDL_Loader_IOR_h
#include "SIDL_Loader_IOR.h"
#endif
// 
// Includes for all method dependencies.
// 
#ifndef included_SIDL_BaseClass_hh
#include "SIDL_BaseClass.hh"
#endif
#ifndef included_SIDL_BaseInterface_hh
#include "SIDL_BaseInterface.hh"
#endif
#ifndef included_SIDL_ClassInfo_hh
#include "SIDL_ClassInfo.hh"
#endif
#ifndef included_SIDL_DLL_hh
#include "SIDL_DLL.hh"
#endif

namespace SIDL { 

  /**
   * Symbol "SIDL.Loader" (version 0.8.2)
   * 
   * Class <code>Loader</code> manages dyanamic loading and symbol name
   * resolution for the SIDL runtime system.  The <code>Loader</code> class
   * manages a library search path and keeps a record of all libraries
   * loaded through this interface, including the initial "global" symbols
   * in the main program.  Unless explicitly set, the search path is taken
   * from the environment variable SIDL_DLL_PATH, which is a semi-colon
   * separated sequence of URIs as described in class <code>DLL</code>.
   */
  class Loader : public ::SIDL::StubBase {

    //////////////////////////////////////////////////
    // 
    // User Defined Methods
    // 

  public:

    /**
     * Set the search path, which is a semi-colon separated sequence of
     * URIs as described in class <code>DLL</code>.  This method will
     * invalidate any existing search path.
     */
    static void
    setSearchPath (
      /*in*/ const ::std::string& path_name
    )
    throw ( ::SIDL::NullIORException ) 
    ;



    /**
     * Return the current search path.  If the search path has not been
     * set, then the search path will be taken from environment variable
     * SIDL_DLL_PATH.
     */
    static ::std::string
    getSearchPath() throw ( ::SIDL::NullIORException ) 
    ;


    /**
     * Append the specified path fragment to the beginning of the
     * current search path.  If the search path has not yet been set
     * by a call to <code>setSearchPath</code>, then this fragment will
     * be appended to the path in environment variable SIDL_DLL_PATH.
     */
    static void
    addSearchPath (
      /*in*/ const ::std::string& path_fragment
    )
    throw ( ::SIDL::NullIORException ) 
    ;



    /**
     * Load the specified library if it has not already been loaded.
     * The URI format is defined in class <code>DLL</code>.  The search
     * path is not searched to resolve the library name.
     */
    static bool
    loadLibrary (
      /*in*/ const ::std::string& uri
    )
    throw ( ::SIDL::NullIORException ) 
    ;



    /**
     * Append the specified DLL to the beginning of the list of already
     * loaded DLLs.
     */
    static void
    addDLL (
      /*in*/ ::SIDL::DLL dll
    )
    throw ( ::SIDL::NullIORException ) 
    ;



    /**
     * Unload all dynamic link libraries.  The library may no longer
     * be used to access symbol names.  When the library is actually
     * unloaded from the memory image depends on details of the operating
     * system.
     */
    static void
    unloadLibraries() throw ( ::SIDL::NullIORException ) 
    ;


    /**
     * Look up the secified symbol name.  If the symbol name cannot be
     * found in one of the already loaded libraries, then the method will
     * search through the library search path.  A null is returned if the
     * symbol could not be resolved.
     */
    static void*
    lookupSymbol (
      /*in*/ const ::std::string& linker_name
    )
    throw ( ::SIDL::NullIORException ) 
    ;



    /**
     * Create an instance of the specified SIDL class.  If the class
     * constructor cannot be found in one of the already loaded libraries,
     * then the method will search through the library search path.  A null
     * object is returned if the symbol could not be resolved.
     */
    static ::SIDL::BaseClass
    createClass (
      /*in*/ const ::std::string& sidl_name
    )
    throw ( ::SIDL::NullIORException ) 
    ;



    /**
     * <p>
     * Add one to the intrinsic reference count in the underlying object.
     * Object in <code>SIDL</code> have an intrinsic reference count.
     * Objects continue to exist as long as the reference count is
     * positive. Clients should call this method whenever they
     * create another ongoing reference to an object or interface.
     * </p>
     * <p>
     * This does not have a return value because there is no language
     * independent type that can refer to an interface or a
     * class.
     * </p>
     */
    void
    addRef() throw ( ::SIDL::NullIORException ) 
    ;


    /**
     * Decrease by one the intrinsic reference count in the underlying
     * object, and delete the object if the reference is non-positive.
     * Objects in <code>SIDL</code> have an intrinsic reference count.
     * Clients should call this method whenever they remove a
     * reference to an object or interface.
     */
    void
    deleteRef() throw ( ::SIDL::NullIORException ) 
    ;


    /**
     * Return true if and only if <code>obj</code> refers to the same
     * object as this object.
     */
    bool
    isSame (
      /*in*/ ::SIDL::BaseInterface iobj
    )
    throw ( ::SIDL::NullIORException ) 
    ;



    /**
     * Check whether the object can support the specified interface or
     * class.  If the <code>SIDL</code> type name in <code>name</code>
     * is supported, then a reference to that object is returned with the
     * reference count incremented.  The callee will be responsible for
     * calling <code>deleteRef</code> on the returned object.  If
     * the specified type is not supported, then a null reference is
     * returned.
     */
    ::SIDL::BaseInterface
    queryInt (
      /*in*/ const ::std::string& name
    )
    throw ( ::SIDL::NullIORException ) 
    ;



    /**
     * Return whether this object is an instance of the specified type.
     * The string name must be the <code>SIDL</code> type name.  This
     * routine will return <code>true</code> if and only if a cast to
     * the string type name would succeed.
     */
    bool
    isType (
      /*in*/ const ::std::string& name
    )
    throw ( ::SIDL::NullIORException ) 
    ;



    /**
     * Return the meta-data about the class implementing this interface.
     */
    ::SIDL::ClassInfo
    getClassInfo() throw ( ::SIDL::NullIORException ) 
    ;


    //////////////////////////////////////////////////
    // 
    // End User Defined Methods
    // (everything else in this file is specific to
    //  Babel's C++ bindings)
    // 

  public:
    typedef struct SIDL_Loader__object ior_t;
    typedef struct SIDL_Loader__external ext_t;
    typedef struct SIDL_Loader__sepv sepv_t;

    // default constructor
    Loader() : d_self(0), d_weak_reference(false) { }

    // static constructor
    static ::SIDL::Loader _create();

    // default destructor
    virtual ~Loader ();

    // copy constructor
    Loader ( const Loader& original );

    // assignment operator
    Loader& operator= ( const Loader& rhs );

    // conversion from ior to C++ class
    Loader ( Loader::ior_t* ior );

    // Alternate constructor: does not call addRef()
    // (sets d_weak_reference=isWeak)
    // For internal use by Impls (fixes bug#275)
    Loader ( Loader::ior_t* ior, bool isWeak );

    // conversion from a StubBase
    Loader ( const ::SIDL::StubBase& base );

    ior_t* _get_ior() { return d_self; }

    const ior_t* _get_ior() const { return d_self; }

    void _set_ior( ior_t* ptr ) { d_self = ptr; }

    bool _is_nil() const { return (d_self==0); }

    bool _not_nil() const { return (d_self!=0); }

    bool operator !() const { return (d_self==0); }

  protected:
    virtual void* _cast(const char* type) const;

  private:
    // Pointer to SIDL's IOR type (one per instance)
    ior_t * d_self;

    // Weak references (used by Impl's only) don't add/deleteRef()
    bool d_weak_reference;

    // Pointer to external (DLL loadable) symbols (shared among instances)
    static const ext_t * s_ext;

  public:
    static const ext_t * _get_ext() throw ( ::SIDL::NullIORException );

    static const sepv_t * _get_sepv() {
      return (*(_get_ext()->getStaticEPV))();
    }

  }; // end class Loader
} // end namespace SIDL

// array specialization
namespace SIDL {
  template<>
  class array< ::SIDL::Loader> : public array_mixin< struct SIDL_Loader__array,
    struct SIDL_Loader__object*, ::SIDL::Loader>{
  public:
    /**
     * default constructor
     */
    array() : array_mixin< struct SIDL_Loader__array,
      struct SIDL_Loader__object*, ::SIDL::Loader>() {}

    /**
     * default destructor
     */
    virtual ~array() { if ( d_array ) { deleteRef(); } }

    /**
     * copy constructor
     */
    array( const array< ::SIDL::Loader >& original ) {
      d_array = original.d_array;
      if ( d_array ) { addRef(); }
    }

    /**
     * assignment operator
     */
    array< ::SIDL::Loader >& operator=( const array< ::SIDL::Loader >& rhs ) {
      if ( d_array != rhs.d_array ) {
        if ( d_array ) { deleteRef(); }
        d_array=rhs.d_array;
        if ( d_array ) { addRef(); }
      }
      return *this;
    }

  /**
   * conversion from ior to C++ class
   * (constructor/casting operator)
   */
  array( struct SIDL_Loader__array* src ) : array_mixin< struct 
    SIDL_Loader__array, struct SIDL_Loader__object*, ::SIDL::Loader>(src) {}

  /**
   * static constructor: createRow
   */
  static array < ::SIDL::Loader >
  createRow(int32_t dimen, const int32_t lower[], const int32_t upper[]) {
    array < ::SIDL::Loader > a;
    a._set_ior( reinterpret_cast<struct SIDL_Loader__array *>(
                SIDL_interface__array_createRow( dimen, lower, upper ) ) );
    return a;
  }

  /**
   * static constructor: createCol
   */
  static array < ::SIDL::Loader >
  createCol(int32_t dimen, const int32_t lower[], const int32_t upper[]) {
    array < ::SIDL::Loader > a;
    a._set_ior( reinterpret_cast<struct SIDL_Loader__array *>(
                SIDL_interface__array_createCol( dimen, lower, upper ) ) );
    return a;
  }

  /**
   * static constructor: create1d
   */
  static array < ::SIDL::Loader >
  create1d(int32_t len) {
    array < ::SIDL::Loader > a;
    a._set_ior( reinterpret_cast<struct SIDL_Loader__array *>(
                SIDL_interface__array_create1d(len)));
    return a;
  }

  /**
   * static constructor: create2dCol
   */
  static array < ::SIDL::Loader >
  create2dCol(int32_t m, int32_t n) {
    array < ::SIDL::Loader > a;
    a._set_ior( reinterpret_cast<struct SIDL_Loader__array *>(
                SIDL_interface__array_create2dCol(m,n)));
    return a;
  }

  /**
   * static constructor: create2dRow
   */
  static array < ::SIDL::Loader >
  create2dRow(int32_t m, int32_t n) {
    array < ::SIDL::Loader > a;
    a._set_ior( reinterpret_cast<struct SIDL_Loader__array *>(
                SIDL_interface__array_create2dRow(m,n)));
    return a;
  }

  /**
   * constructor: slice
   */
  array < ::SIDL::Loader >
  slice( int dimen,
         const int32_t newElem[],
         const int32_t *srcStart = 0,
         const int32_t *srcStride = 0,
         const int32_t *newStart = 0) {
    array < ::SIDL::Loader > a;
    a._set_ior( reinterpret_cast<struct SIDL_Loader__array *>(
                SIDL_interface__array_slice( reinterpret_cast<struct 
      SIDL_interface__array *>(d_array),
                                   dimen, newElem, srcStart, srcStride,
      newStart) ) );
    return a;
  }

  /**
   * copy
   */
  void copy( const array< ::SIDL::Loader>& src ) {
    SIDL_interface__array_copy( reinterpret_cast<const struct 
      SIDL_interface__array *>( src._get_ior()),
      reinterpret_cast<struct SIDL_interface__array *>(_get_ior()));
  }

  /**
   * constructor: smart copy
   */
  void smartCopy() {
    struct SIDL_interface__array * p  = SIDL_interface__array_smartCopy(
      reinterpret_cast<struct SIDL_interface__array *>(_get_ior()));
    if ( _not_nil() ) { deleteRef(); }
    _set_ior( reinterpret_cast<struct SIDL_Loader__array*>(p));
  }

  /**
   * constructor: ensure
   */
  void ensure(int32_t dimen, int ordering ) {
    struct SIDL_interface__array * p  = SIDL_interface__array_ensure(
      reinterpret_cast<struct SIDL_interface__array *>(d_array), dimen,
      ordering );
    if ( _not_nil() ) { deleteRef(); }
    _set_ior( reinterpret_cast<struct SIDL_Loader__array*>(p));
  }
};
}

#endif
