// 
// File:          sidl_BaseInterface.hh
// Symbol:        sidl.BaseInterface-v0.9.0
// Symbol Type:   interface
// Babel Version: 0.9.3
// Release:       $Name:  $
// Revision:      @(#) $Id: $
// Description:   Client-side glue code for sidl.BaseInterface
// 
// Copyright (c) 2000-2002, The Regents of the University of California.
// Produced at the Lawrence Livermore National Laboratory.
// Written by the Components Team <components@llnl.gov>
// All rights reserved.
// 
// This file is part of Babel. For more information, see
// http://www.llnl.gov/CASC/components/. Please read the COPYRIGHT file
// for Our Notice and the LICENSE file for the GNU Lesser General Public
// License.
// 
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License (as published by
// the Free Software Foundation) version 2.1 dated February 1999.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms and
// conditions of the GNU Lesser General Public License for more details.
// 
// You should have recieved a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software Foundation,
// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
// 
// WARNING: Automatically generated; changes will be lost
// 
// babel-version = 0.9.3
// 

#ifndef included_sidl_BaseInterface_hh
#define included_sidl_BaseInterface_hh

// declare class before #includes
// (this alleviates circular #include guard problems)[BUG#393]
namespace sidl { 

  class BaseInterface;
} // end namespace sidl

// Some compilers need to define array template before the specializations
#ifndef included_sidl_cxx_hh
#include "sidl_cxx.hh"
#endif
namespace sidl {
  template<>
  class array< ::sidl::BaseInterface >;
}

#ifndef included_sidl_cxx_hh
#include "sidl_cxx.hh"
#endif
#ifndef included_sidl_BaseInterface_IOR_h
#include "sidl_BaseInterface_IOR.h"
#endif
// 
// Includes for all method dependencies.
// 
#ifndef included_sidl_ClassInfo_hh
#include "sidl_ClassInfo.hh"
#endif

namespace sidl { 

  /**
   * Symbol "sidl.BaseInterface" (version 0.9.0)
   * 
   * Every interface in <code>sidl</code> implicitly inherits
   * from <code>BaseInterface</code>, and it is implemented
   * by <code>BaseClass</code> below.
   */
  class BaseInterface : public ::sidl::StubBase {

    //////////////////////////////////////////////////
    // 
    // User Defined Methods
    // 

  public:

    /**
     * <p>
     * Add one to the intrinsic reference count in the underlying object.
     * Object in <code>sidl</code> have an intrinsic reference count.
     * Objects continue to exist as long as the reference count is
     * positive. Clients should call this method whenever they
     * create another ongoing reference to an object or interface.
     * </p>
     * <p>
     * This does not have a return value because there is no language
     * independent type that can refer to an interface or a
     * class.
     * </p>
     */
    void
    addRef() throw ( ::sidl::NullIORException ) 
    ;


    /**
     * Decrease by one the intrinsic reference count in the underlying
     * object, and delete the object if the reference is non-positive.
     * Objects in <code>sidl</code> have an intrinsic reference count.
     * Clients should call this method whenever they remove a
     * reference to an object or interface.
     */
    void
    deleteRef() throw ( ::sidl::NullIORException ) 
    ;


    /**
     * Return true if and only if <code>obj</code> refers to the same
     * object as this object.
     */
    bool
    isSame (
      /*in*/ ::sidl::BaseInterface iobj
    )
    throw ( ::sidl::NullIORException ) 
    ;



    /**
     * Check whether the object can support the specified interface or
     * class.  If the <code>sidl</code> type name in <code>name</code>
     * is supported, then a reference to that object is returned with the
     * reference count incremented.  The callee will be responsible for
     * calling <code>deleteRef</code> on the returned object.  If
     * the specified type is not supported, then a null reference is
     * returned.
     */
    ::sidl::BaseInterface
    queryInt (
      /*in*/ const ::std::string& name
    )
    throw ( ::sidl::NullIORException ) 
    ;



    /**
     * Return whether this object is an instance of the specified type.
     * The string name must be the <code>sidl</code> type name.  This
     * routine will return <code>true</code> if and only if a cast to
     * the string type name would succeed.
     */
    bool
    isType (
      /*in*/ const ::std::string& name
    )
    throw ( ::sidl::NullIORException ) 
    ;



    /**
     * Return the meta-data about the class implementing this interface.
     */
    ::sidl::ClassInfo
    getClassInfo() throw ( ::sidl::NullIORException ) 
    ;


    //////////////////////////////////////////////////
    // 
    // End User Defined Methods
    // (everything else in this file is specific to
    //  Babel's C++ bindings)
    // 

  public:
    typedef struct sidl_BaseInterface__object ior_t;
    typedef struct sidl_BaseInterface__external ext_t;
    typedef struct sidl_BaseInterface__sepv sepv_t;

    // default constructor
    BaseInterface() : d_self(0), d_weak_reference(false) { }

    // default destructor
    virtual ~BaseInterface ();

    // copy constructor
    BaseInterface ( const BaseInterface& original );

    // assignment operator
    BaseInterface& operator= ( const BaseInterface& rhs );

    // conversion from ior to C++ class
    BaseInterface ( BaseInterface::ior_t* ior );

    // Alternate constructor: does not call addRef()
    // (sets d_weak_reference=isWeak)
    // For internal use by Impls (fixes bug#275)
    BaseInterface ( BaseInterface::ior_t* ior, bool isWeak );

    // conversion from a StubBase
    BaseInterface ( const ::sidl::StubBase& base );

    ior_t* _get_ior() { return d_self; }

    const ior_t* _get_ior() const { return d_self; }

    void _set_ior( ior_t* ptr ) { d_self = ptr; }

    bool _is_nil() const { return (d_self==0); }

    bool _not_nil() const { return (d_self!=0); }

    bool operator !() const { return (d_self==0); }

  protected:
    virtual void* _cast(const char* type) const;

  private:
    // Pointer to sidl's IOR type (one per instance)
    ior_t * d_self;

    // Weak references (used by Impl's only) don't add/deleteRef()
    bool d_weak_reference;

    // Pointer to external (DLL loadable) symbols (shared among instances)
    static const ext_t * s_ext;

  public:
    static const ext_t * _get_ext() throw ( ::sidl::NullIORException );

  }; // end class BaseInterface
} // end namespace sidl

namespace sidl {
  // traits specialization
  template<>
  struct array_traits< ::sidl::BaseInterface > {
    typedef array< ::sidl::BaseInterface > cxx_array_t;
    typedef ::sidl::BaseInterface cxx_item_t;
    typedef struct sidl_BaseInterface__array ior_array_t;
    typedef sidl_interface__array ior_array_internal_t;
    typedef struct sidl_BaseInterface__object ior_item_t;
    typedef cxx_item_t value_type;
    typedef value_type reference;
    typedef value_type* pointer;
    typedef const value_type const_reference;
    typedef const value_type* const_pointer;
    typedef array_iter< array_traits< ::sidl::BaseInterface > > iterator;
    typedef const_array_iter< array_traits< ::sidl::BaseInterface > > 
      const_iterator;
  };

  // array specialization
  template<>
  class array< ::sidl::BaseInterface >: public interface_array< array_traits< 
    ::sidl::BaseInterface > > {
  public:
    typedef interface_array< array_traits< ::sidl::BaseInterface > > Base;
    typedef array_traits< ::sidl::BaseInterface >::cxx_array_t          
      cxx_array_t;
    typedef array_traits< ::sidl::BaseInterface >::cxx_item_t           
      cxx_item_t;
    typedef array_traits< ::sidl::BaseInterface >::ior_array_t          
      ior_array_t;
    typedef array_traits< ::sidl::BaseInterface >::ior_array_internal_t 
      ior_array_internal_t;
    typedef array_traits< ::sidl::BaseInterface >::ior_item_t           
      ior_item_t;

    /**
     * conversion from ior to C++ class
     * (constructor/casting operator)
     */
    array( struct sidl_BaseInterface__array* src = 0) : Base(src) {}

    /**
     * copy constructor
     */
    array( const array< ::sidl::BaseInterface >&src) {
      d_array = src.d_array;
      if (d_array) addRef();
    }

    /**
     * assignment
     */
    array< ::sidl::BaseInterface >&
    operator =( const array< ::sidl::BaseInterface >&rhs ) { 
      if (d_array != rhs.d_array) {
        if (d_array) deleteRef();
        d_array = rhs.d_array;
        if (d_array) addRef();
      }
      return *this;
    }

  };
}

#endif
