// 
// File:          sidl_Loader.cc
// Symbol:        sidl.Loader-v0.9.0
// Symbol Type:   class
// Babel Version: 0.9.3
// Release:       $Name:  $
// Revision:      @(#) $Id: $
// Description:   Client-side glue code for sidl.Loader
// 
// Copyright (c) 2000-2002, The Regents of the University of California.
// Produced at the Lawrence Livermore National Laboratory.
// Written by the Components Team <components@llnl.gov>
// All rights reserved.
// 
// This file is part of Babel. For more information, see
// http://www.llnl.gov/CASC/components/. Please read the COPYRIGHT file
// for Our Notice and the LICENSE file for the GNU Lesser General Public
// License.
// 
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License (as published by
// the Free Software Foundation) version 2.1 dated February 1999.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms and
// conditions of the GNU Lesser General Public License for more details.
// 
// You should have recieved a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software Foundation,
// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
// 
// WARNING: Automatically generated; changes will be lost
// 
// babel-version = 0.9.3
// 

#ifndef included_sidl_Loader_hh
#include "sidl_Loader.hh"
#endif
#ifndef included_sidl_BaseInterface_hh
#include "sidl_BaseInterface.hh"
#endif
#ifndef included_sidl_BaseClass_hh
#include "sidl_BaseClass.hh"
#endif
#include "sidl_String.h"


//////////////////////////////////////////////////
// 
// User Defined Methods
// 


/**
 * Set the search path, which is a semi-colon separated sequence of
 * URIs as described in class <code>DLL</code>.  This method will
 * invalidate any existing search path.
 */
void
sidl::Loader::setSearchPath( /*in*/ const ::std::string& path_name )
throw ( ::sidl::NullIORException ) 

{

  // pack args to dispatch to ior

  // dispatch to ior
  ( _get_sepv()->f_setSearchPath)( /* in */ path_name.c_str() );
  // unpack results and cleanup

}



/**
 * Return the current search path.  If the search path has not been
 * set, then the search path will be taken from environment variable
 * SIDL_DLL_PATH.
 */
::std::string
sidl::Loader::getSearchPath(  )
throw ( ::sidl::NullIORException ) 

{
  ::std::string _result;
  // pack args to dispatch to ior
  char * _local_result;
  // dispatch to ior
  _local_result = ( _get_sepv()->f_getSearchPath)(  );
  // unpack results and cleanup
  if (_local_result) {
    _result = _local_result;
    free( _local_result );
  }
  return _result;
}



/**
 * Append the specified path fragment to the beginning of the
 * current search path.  If the search path has not yet been set
 * by a call to <code>setSearchPath</code>, then this fragment will
 * be appended to the path in environment variable SIDL_DLL_PATH.
 */
void
sidl::Loader::addSearchPath( /*in*/ const ::std::string& path_fragment )
throw ( ::sidl::NullIORException ) 

{

  // pack args to dispatch to ior

  // dispatch to ior
  ( _get_sepv()->f_addSearchPath)( /* in */ path_fragment.c_str() );
  // unpack results and cleanup

}



/**
 * Load the specified library if it has not already been loaded.
 * The URI format is defined in class <code>DLL</code>.  The search
 * path is not searched to resolve the library name.
 * 
 * @param uri          the URI to load. This can be a .la file
 *                     (a metadata file produced by libtool) or
 *                     a shared library binary (i.e., .so,
 *                     .dll or whatever is appropriate for your
 *                     OS)
 * @param loadGlobally <code>true</code> means that the shared
 *                     library symbols will be loaded into the
 *                     global namespace; <code>false</code> 
 *                     means they will be loaded into a 
 *                     private namespace. Some operating systems
 *                     may not be able to honor the value presented
 *                     here.
 * @param loadLazy     <code>true</code> instructs the loader to
 *                     that symbols can be resolved as needed (lazy)
 *                     instead of requiring everything to be resolved
 *                     now.
 * @return if the load was successful, a non-NULL DLL object is returned.
 */
::sidl::DLL
sidl::Loader::loadLibrary( /*in*/ const ::std::string& uri,
  /*in*/ bool loadGlobally, /*in*/ bool loadLazy )
throw ( ::sidl::NullIORException ) 

{
  ::sidl::DLL _result;
  // pack args to dispatch to ior
  sidl_bool _local_loadGlobally = loadGlobally;
  sidl_bool _local_loadLazy = loadLazy;
  // dispatch to ior
  _result = ::sidl::DLL( ( _get_sepv()->f_loadLibrary)( /* in */ uri.c_str(),
    /* in */ _local_loadGlobally, /* in */ _local_loadLazy ), false);
  // unpack results and cleanup

  return _result;
}



/**
 * Append the specified DLL to the beginning of the list of already
 * loaded DLLs.
 */
void
sidl::Loader::addDLL( /*in*/ ::sidl::DLL dll )
throw ( ::sidl::NullIORException ) 

{

  // pack args to dispatch to ior

  // dispatch to ior
  ( _get_sepv()->f_addDLL)( /* in */ dll._get_ior() );
  // unpack results and cleanup

}



/**
 * Unload all dynamic link libraries.  The library may no longer
 * be used to access symbol names.  When the library is actually
 * unloaded from the memory image depends on details of the operating
 * system.
 */
void
sidl::Loader::unloadLibraries(  )
throw ( ::sidl::NullIORException ) 

{

  // pack args to dispatch to ior

  // dispatch to ior
  ( _get_sepv()->f_unloadLibraries)(  );
  // unpack results and cleanup

}



/**
 * Find a DLL containing the specified information for a sidl
 * class. This method searches SCL files in the search path looking
 * for a shared library that contains the client-side or IOR
 * for a particular sidl class.
 * 
 * @param sidl_name  the fully qualified (long) name of the
 *                   class/interface to be found. Package names
 *                   are separated by period characters from each
 *                   other and the class/interface name.
 * @param target     to find a client-side binding, this is
 *                   normally the name of the language.
 *                   To find the implementation of a class
 *                   in order to make one, you should pass
 *                   the string "ior/impl" here.
 * @param lScope     this specifies whether the symbols should
 *                   be loaded into the global scope, a local
 *                   scope, or use the setting in the SCL file.
 * @param lResolve   this specifies whether symbols should be
 *                   resolved as needed (LAZY), completely
 *                   resolved at load time (NOW), or use the
 *                   setting from the SCL file.
 * @return a non-NULL object means the search was successful.
 *         The DLL has already been added.
 */
::sidl::DLL
sidl::Loader::findLibrary( /*in*/ const ::std::string& sidl_name,
  /*in*/ const ::std::string& target, /*in*/ ::sidl::Scope lScope,
  /*in*/ ::sidl::Resolve lResolve )
throw ( ::sidl::NullIORException ) 

{
  ::sidl::DLL _result;
  // pack args to dispatch to ior

  // dispatch to ior
  _result = ::sidl::DLL( ( _get_sepv()->f_findLibrary)( /* in */ 
    sidl_name.c_str(), /* in */ target.c_str(),
    /* in */ (enum sidl_Scope__enum)lScope,
    /* in */ (enum sidl_Resolve__enum)lResolve ), false);
  // unpack results and cleanup

  return _result;
}



/**
 * <p>
 * Add one to the intrinsic reference count in the underlying object.
 * Object in <code>sidl</code> have an intrinsic reference count.
 * Objects continue to exist as long as the reference count is
 * positive. Clients should call this method whenever they
 * create another ongoing reference to an object or interface.
 * </p>
 * <p>
 * This does not have a return value because there is no language
 * independent type that can refer to an interface or a
 * class.
 * </p>
 */
void
sidl::Loader::addRef(  )
throw ( ::sidl::NullIORException ) 

{
  if ( d_self == 0 ) {
    throw ::sidl::NullIORException( ::std::string (
      "Null IOR Pointer in \"sidl::Loader::addRef()\""
    ));
  }

  if ( !d_weak_reference ) {
    // pack args to dispatch to ior

    // dispatch to ior
    (*(d_self->d_epv->f_addRef))(d_self );
    // unpack results and cleanup

  }
}



/**
 * Decrease by one the intrinsic reference count in the underlying
 * object, and delete the object if the reference is non-positive.
 * Objects in <code>sidl</code> have an intrinsic reference count.
 * Clients should call this method whenever they remove a
 * reference to an object or interface.
 */
void
sidl::Loader::deleteRef(  )
throw ( ::sidl::NullIORException ) 

{
  if ( d_self == 0 ) {
    throw ::sidl::NullIORException( ::std::string (
      "Null IOR Pointer in \"sidl::Loader::deleteRef()\""
    ));
  }

  if ( !d_weak_reference ) {
    // pack args to dispatch to ior

    // dispatch to ior
    (*(d_self->d_epv->f_deleteRef))(d_self );
    // unpack results and cleanup

    d_self = 0;
  }
}



/**
 * Return true if and only if <code>obj</code> refers to the same
 * object as this object.
 */
bool
sidl::Loader::isSame( /*in*/ ::sidl::BaseInterface iobj )
throw ( ::sidl::NullIORException ) 

{
  if ( d_self == 0 ) {
    throw ::sidl::NullIORException( ::std::string (
      "Null IOR Pointer in \"sidl::Loader::isSame()\""
    ));
  }
  bool _result;
  // pack args to dispatch to ior
  sidl_bool _local_result;
  // dispatch to ior
  _local_result = (*(d_self->d_epv->f_isSame))(d_self,
    /* in */ iobj._get_ior() );
  // unpack results and cleanup
  _result = _local_result;
  return _result;
}



/**
 * Check whether the object can support the specified interface or
 * class.  If the <code>sidl</code> type name in <code>name</code>
 * is supported, then a reference to that object is returned with the
 * reference count incremented.  The callee will be responsible for
 * calling <code>deleteRef</code> on the returned object.  If
 * the specified type is not supported, then a null reference is
 * returned.
 */
::sidl::BaseInterface
sidl::Loader::queryInt( /*in*/ const ::std::string& name )
throw ( ::sidl::NullIORException ) 

{
  if ( d_self == 0 ) {
    throw ::sidl::NullIORException( ::std::string (
      "Null IOR Pointer in \"sidl::Loader::queryInt()\""
    ));
  }
  ::sidl::BaseInterface _result;
  // pack args to dispatch to ior

  // dispatch to ior
  _result = ::sidl::BaseInterface( (*(d_self->d_epv->f_queryInt))(d_self,
    /* in */ name.c_str() ), false);
  // unpack results and cleanup

  return _result;
}



/**
 * Return whether this object is an instance of the specified type.
 * The string name must be the <code>sidl</code> type name.  This
 * routine will return <code>true</code> if and only if a cast to
 * the string type name would succeed.
 */
bool
sidl::Loader::isType( /*in*/ const ::std::string& name )
throw ( ::sidl::NullIORException ) 

{
  if ( d_self == 0 ) {
    throw ::sidl::NullIORException( ::std::string (
      "Null IOR Pointer in \"sidl::Loader::isType()\""
    ));
  }
  bool _result;
  // pack args to dispatch to ior
  sidl_bool _local_result;
  // dispatch to ior
  _local_result = (*(d_self->d_epv->f_isType))(d_self, /* in */ name.c_str() );
  // unpack results and cleanup
  _result = _local_result;
  return _result;
}



/**
 * Return the meta-data about the class implementing this interface.
 */
::sidl::ClassInfo
sidl::Loader::getClassInfo(  )
throw ( ::sidl::NullIORException ) 

{
  if ( d_self == 0 ) {
    throw ::sidl::NullIORException( ::std::string (
      "Null IOR Pointer in \"sidl::Loader::getClassInfo()\""
    ));
  }
  ::sidl::ClassInfo _result;
  // pack args to dispatch to ior

  // dispatch to ior
  _result = ::sidl::ClassInfo( (*(d_self->d_epv->f_getClassInfo))(d_self ),
    false);
  // unpack results and cleanup

  return _result;
}



//////////////////////////////////////////////////
// 
// End User Defined Methods
// (everything else in this file is specific to
//  Babel's C++ bindings)
// 

// static constructor
::sidl::Loader
sidl::Loader::_create() {
  ::sidl::Loader self( (*_get_ext()->createObject)(), false );
  return self;
}

// default destructor
sidl::Loader::~Loader () {
  if ( d_self != 0 ) {
    deleteRef();
  }
}

// copy constructor
sidl::Loader::Loader ( const ::sidl::Loader& original ) {
  d_self = const_cast< ior_t*>(original.d_self);
  d_weak_reference = original.d_weak_reference;
  if (d_self != 0 ) {
    addRef();
  }
}

// assignment operator
::sidl::Loader&
sidl::Loader::operator=( const ::sidl::Loader& rhs ) {
  if ( d_self != rhs.d_self ) {
    if ( d_self != 0 ) {
      deleteRef();
    }
    d_self = const_cast< ior_t*>(rhs.d_self);
    d_weak_reference = rhs.d_weak_reference;
    if ( d_self != 0 ) {
      addRef();
    }
  }
  return *this;
}

// conversion from ior to C++ class
sidl::Loader::Loader ( ::sidl::Loader::ior_t* ior ) 
    : d_self( ior ), d_weak_reference(false) {
  if ( d_self != 0 ) {
    addRef();
  }
}

// Alternate constructor: does not call addRef()
// (sets d_weak_reference=isWeak)
// For internal use by Impls (fixes bug#275)
sidl::Loader::Loader ( ::sidl::Loader::ior_t* ior, bool isWeak ) 
    : d_self( ior ), d_weak_reference(isWeak) { 
}

// conversion from a StubBase
sidl::Loader::Loader ( const ::sidl::StubBase& base )
{
  d_self = reinterpret_cast< ior_t*>(base._cast("sidl.Loader"));
  d_weak_reference = false;
  if (d_self != 0) {
    addRef();
  }
}

// protected method that implements casting
void* sidl::Loader::_cast(const char* type) const
{
  void* ptr = 0;
  if ( d_self != 0 ) {
    ptr = reinterpret_cast< void*>((*d_self->d_epv->f__cast)(d_self, type));
  }
  return ptr;
}

// Static data type
const ::sidl::Loader::ext_t * sidl::Loader::s_ext = 0;

// private static method to get static data type
const ::sidl::Loader::ext_t *
sidl::Loader::_get_ext()
  throw (::sidl::NullIORException)
{
  if (! s_ext ) {
    s_ext = sidl_Loader__externals();
  }
  return s_ext;
}

